"""
Copyright 2008-2014 by Aerospike, Inc. All rights reserved.
THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE. THE COPYRIGHT NOTICE
ABOVE DOES NOT EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION.
"""


# -*- coding: utf-8 -*-
from constants import AMC_PATH, ENTERPRISE_FLAG_FILE
from constants import AMC_CONFIG_PATH
from constants import SECRET_KEY, ALLOWED_ROLES
import sys, datetime, os.path
sys.path.insert(1,'%s/site-packages/' % AMC_PATH)
sys.path.append(str(os.path.dirname(AMC_CONFIG_PATH)))

from flask import Flask
from libs.flask_compress import Compress
from utilities import background_tasks, config_manager
from werkzeug.exceptions import Unauthorized
import traceback
from flask_kvsession import KVSessionExtension
from simplekv.memory import DictStore

store = DictStore()
app = Flask(__name__)
#app.config["UPDATE_INTERVAL"] = int(config_manager.get_update_interval())
app.secret_key = SECRET_KEY
app.config["COMPRESS_DEBUG"] = True
app.config["COMPRESS_MIMETYPES"] = ["text/javascript", "application/javascript", "application/x-javascript", "image/png", "text/html", "text/css"]
KVSessionExtension(store, app)

try:
    from gunicorn_config import certfile, keyfile
    if os.path.exists(certfile) and os.path.exists(keyfile):
        app.config['SESSION_COOKIE_SECURE'] = True
        app.config['SESSION_COOKIE_HTTPONLY'] = False
except ImportError:
    app.config['SESSION_COOKIE_SECURE'] = False
    app.config['SESSION_COOKIE_HTTPONLY'] = True

Compress(app)

from flask import json, request, render_template, session
from controllers import cluster_controller, namespace_controller
from controllers import config_controller, xdr_controller
from controllers import node_controller, udf_controller
from utilities.cluster_manager import ClusterManager
from utilities.cluster_manager import ClusterManager
from utilities.user_manager import UserManager
from utilities.session_manager import SessionManager
from utilities.config_manager import config_manager
from constants import *
from itsdangerous import URLSafeTimedSerializer
from models.user import User
from helpers import encryption
from libs import base
import eventlet
import uuid

ansible_present = True
try:
    import ansible.runner
    from ansible.errors import AnsibleError
except ImportError:
    ansible_present = False

cluster_manager = ClusterManager()
session_manager = SessionManager()
user_manager = UserManager()


# URLs for static contents


@app.route("/", methods=["GET"])
def onepage():

    response = app.make_response(render_template('onepage.html'))
    response.headers.add('amc-page', 'homepage')
    return response


# URLs for apis
#@app.route("/logout/<cluster>", methods=["GET"])
@app.route("/session-terminate", methods=["POST"])
#@login_required
@session_manager.check_session_validity(session)
def logout_page():

    #cluster.remove_user(current_user)
    session_key = session['session']
    session_manager.delete_session(session_key)
    session.pop('session', None)
    response = {'status': 'success'}
    return json.dumps(response)



@app.route("/change_password", methods=["POST"])
#@login_required
@session_manager.check_session_validity(session)
def change_password():

    if 'username' in request.form:
        user = User.get(request.form['username'])
    else:
        user = User.get("admin")

    if user and encryption.hash_pass(request.form['old_password'], app.secret_key) == user.password:
        current_users = UserManager()
        status = current_users.change_password(user.id, encryption.hash_pass(request.form['new_password'], app.secret_key))
        if status == "success":
            response = {'status': "success"}
            return json.dumps(response)
    response = {'status': "failure"}
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/get-current-user", methods=["GET"])
def get_current_user(cluster):

    if 'session' in session:
        session_key = session['session']
        username = session_manager.get_user(session_key, cluster)
        #username = current_user.get_id()

        response = dict(username=username)
        response['cluster_id'] = cluster
    else:
        response = dict(error = 'Session Invalidated!!', username = None, cluster_id = None)
    return json.dumps(response)


@app.route("/set-update-interval/<cluster>", methods=["POST"])
@session_manager.check_session_validity(session,['sys-admin'])
@cluster_manager.check_cluster_validity()
#@user_manager.allowed_for_roles("sys-admin")
def set_update_interval(cluster, user, password):

    if int(request.form['update_interval']) < 1 or int(request.form['update_interval']) > 10:
        response = dict(status="invalid")
    else:
        response = config_manager.set_update_interval(request.form['update_interval'])
    return json.dumps(response)


@app.route("/get_amc_version", methods=["GET"])
def get_amc_version():

    try:
        f = open('/opt/amc/amc_version', 'r')
        amc_version = f.read()
        f.close()
    except:
        amc_version = "Error reading AMC version"
    amc_type = "enterprise" if os.path.isfile(ENTERPRISE_FLAG_FILE) else "community"
    return json.dumps(dict(amc_version=amc_version.strip(), amc_type=amc_type))


@app.route("/aerospike/service/clusters/get-cluster-id", methods=["POST"])
def get_cluster_id():

    node_address = request.form.get('seed_node', '')
    import sys
    #sys.stderr.write('\nIn get_cluster_id node_address:' + str(node_address))
    ip, port = node_address.split(":")
    #session_key = session.get('session', None)
    """if ip == 'localhost':
        sys.stderr.write('\nIn get_cluster_id ip:' + str(ip) + '\t port: ' + str(port))
        node_address = base.get_self_address(ip, port)
        sys.stderr.write('\nIn get_cluster_id node_address from base:' + str(node_address))"""
    """if ip == 'localhost':
        sys.stderr.write('\nIn get_cluster_id ip:' + str(ip) + '\t port: ' + str(port))
        node_address = base.get_self_address(ip, port, user ='', password='')
        sys.stderr.write('\nIn get_cluster_id node_address from base:' + str(node_address))
        if type(node_address) is int:
            node_address = request.form.get('seed_node', '')
            ip, port = node_address.split(":")
        #response = cluster_manager.get_cluster_obj_by_node(node_address)
        #sys.stderr.write('\nRESPONSE RETURNED CLUSTER OBJ BY NODE LOCALHOST: ' + str(cluster_id) + '\nRESPONSE:'  + str(response))
    """
    cluster_name = request.form.get('cluster_name')
    if 'cluster_name' not in request.form:
        cluster_name = None
    from libs import citrusleaf as cl
    #sys.stderr.write('\ncluster_name: ' + str(cluster_name))
    session_key = session.get('session', None)
    #sys.stderr.write('\nSESSION KEY: ' + str(session_key))
    cluster_id = None
    #response = cluster_manager.get_cluster_info(node_address, user='', password='')
    response = cluster_manager.get_cluster_obj_by_node(node_address)
    #sys.stderr.write('\nRESPONSE RETURNED CLUSTER OBJ BY NODE AT FIRST: ' + str(cluster_id) + '\nRESPONSE:'  + str(response))
    if response is not None and session_key is not None:
        #sys.stderr.write('\nCLUSTER PRESENT: ' + str(session_manager.is_cluster_present(session_key, node_address, cluster_name)))
        resp =  cluster_manager.get_cluster_data_by_cluster_object(response)
        #sys.stderr.write('\nREPONSE FROM OBJ!!!!!: ' + str(resp))

        if session_manager.is_cluster_present(session_key, node_address, cluster_name) is not False:
            #resp =  cluster_manager.get_cluster_data_by_cluster_object(response)
            #sys.stderr.write('\nREPONSE FROM OBJ!!!!!: ' + str(resp))
            resp['status'] = 'success'
            if resp['seed_address'] != node_address :
                if node_address in resp['nodes']:
                    #sys.stderr.write('\nCHANGING SEED NODE FROM: ' + str(resp['seed_address']) + '\tTO:' + str(node_address))
                    resp['seed_address'] = str(node_address)
            return json.dumps(resp)
        elif node_address in resp['nodes'] and session_manager.is_cluster_id_present(session_key, resp['cluster_id']):
            #sys.stderr.write('\nNODE ADDRESS PRESENT IN CLUSTER OBJ and ALSO CLUSTER ID IN SESSION!!!!!')
            resp['status'] = 'success'
            if resp['seed_address'] != node_address :
                if node_address in resp['nodes']:
                    #sys.stderr.write('\nCHANGING SEED NODE FROM: ' + str(resp['seed_address']) + '\tTO:' + str(node_address))
                    resp['seed_address'] = str(node_address)
            return json.dumps(resp)

    if session_manager.is_valid_session(session_key):
        cluster_id = session_manager.is_cluster_present(session_key, node_address, cluster_name)
    #sys.stderr.write('\n Is valid session in flaskapp cluster_id:' + str(cluster_id))
    if cluster_id:
        """old_cluster_name = session_manager.get_cluster_name(session_key, cluster_id)
        if cluster_name is not None and cluster_name != old_cluster_name:
            session_manager.update_cluster_name(session_key, cluster_id, cluster_name)"""
        response = cluster_manager.get_cluster_data_by_cluster_id(cluster_id)
        response['status'] = 'success'
        #sys.stderr.write('\nRESPONSE FROM CLUSTER ID: ' + str(response))
        return json.dumps(response)
    elif 'username' in request.form and 'password' in request.form:
        response = cluster_manager.get_cluster_info(node_address,
                                    user=str(request.form['username']), password=cl.hashpassword(str(request.form['password'])))
        #sys.stderr.write('\nRESPONSE FROM ELIF CLUSTER ID: ' + str(response))
    else:
        response = cluster_manager.get_cluster_info(node_address, user='', password='')
        #sys.stderr.write('\nRESPONSE FROM ELSE  CLUSTER ID: ' + str(response))
    #sys.stderr.write('\nRESPONSE RETURNED CLUSTER INFO: ' + str(cluster_id) + '\nRESPONSE:'  + str(response))
    if 'error' not in response:
        if response['cluster_id'] is not None:
            response['status'] = 'success'

            if not session_manager.is_valid_session(session_key):
                session_key = uuid.uuid4()
                session['session'] = session_key
                session_manager.create_session(session_key)
            if 'username' in request.form:

                s_ip, s_port = str(response['seed_address']).split(":")
                #sys.stderr.write('\n\nsip port:'+ str(s_ip) + str(s_port))
                user_roles = base.get_cluster_userroles(s_ip, s_port,user=str(request.form['username']),
                     password=str(cl.hashpassword(str(request.form['password']))))

                if 'sys-admin' in user_roles:
                    #sys.stderr.write('\n\nADDING CLUSTER SECURED!')
                    session_manager.add_cluster(session['session'],response['seed_address'],
                        response['cluster_id'],
                        cluster_name,user=str(request.form['username']),
                        password= cl.hashpassword(str(request.form['password'])),
                        pl_password = str(request.form['password']))
                    import time
                    import datetime
                    ts = time.time()
                    st = datetime.datetime.fromtimestamp(ts).strftime('%Y-%m-%d %H:%M:%S')
                    sys.stderr.write('\n' + str(st) + ' : Logging in..')
                else:
                    response['status'] = 'failure'
                    response['error'] = "You don't have enough permissions"

            else:
                    session_manager.add_cluster(session['session'],node_address,response['cluster_id'],cluster_name)
                    #sys.stderr.write('\n\nADDING CLUSTER ELSE!')
    else:
        response['status'] = 'failure'
    return json.dumps(response)

@app.route("/get_current_monitoring_clusters", methods=["GET"])
@session_manager.check_session_validity(session)
def get_current_monitoring_clusters():
    session_key = session['session']
    response = session_manager.get_clusters_from_session(session_key)
    return json.dumps(response)

@app.route("/aerospike/service/clusters/<cluster>/logout", methods=["POST"])
@session_manager.check_session_validity(session)
def delete_cluster_from_session(cluster, user, password):
    session_key = session['session']
    status = session_manager.remove_cluster(session_key,cluster)
    if status:
        response = dict(status="success")
    else:
        response = dict(status="failure")
    return json.dumps(response)

@app.route("/aerospike/service/clusters/<cluster>", methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
def get_cluster_info(cluster, user, password):

    response = cluster_controller.get_info(cluster, user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/basic", methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
def get_cluster_basic_info(cluster, user, password):

    response = cluster_controller.get_basic_info(cluster, user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/nodes/<node_list>", methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
def get_node_info(cluster, node_list, user, password):

    response = node_controller.get_info(cluster, node_list,
                                    user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/nodes/<address>/allstats", methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
def get_node_stats(cluster, address, user, password):

    stats_type = request.args.get("type", "all")

    response = node_controller.get_stats(cluster, address, stats_type,
                                user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/nodes/<address>/allconfig", methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
def get_node_config(cluster, address, user, password):

    response = node_controller.get_all_config(cluster, address)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/nodes/<node_list>/setconfig", methods=["POST"])
@session_manager.check_session_validity(session,['sys-admin'])
@cluster_manager.check_cluster_validity()
#@user_manager.allowed_for_roles("sys-admin")
def set_node_config(cluster, node_list, user, password):

    edit_parameters = request.form
    import sys
    #sys.stderr.write('\nUser n pwd in set_node config flaskapp: ' + str(user) + '\t' + str(password))
    response = config_controller.set_node_config(cluster, node_list, edit_parameters,
                                                user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace>/nodes/<node_list>",
    methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
def get_namespace_info_from_node(cluster, namespace, node_list, user, password):

    response = namespace_controller.get_namespace_info_per_node(cluster, str(namespace), node_list,
                                            user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace>/nodes/<node>/allstats",
    methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
def get_namespace_stats_from_node(cluster, namespace, node, user, password):

    stats_type = request.args.get("type", "all")

    response = namespace_controller.get_stats(cluster, str(namespace), node, stats_type,
                                                        user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace_list>", methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
def get_namespace_info(cluster, namespace_list, user, password):

    node_addresses = request.args.get('nodes', None)

    response = namespace_controller.get_namespace_info(cluster, str(namespace_list), node_addresses,
                                                user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace>/nodes/<node>/allconfig",
    methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
def get_namespace_config(cluster, namespace, node, user, password):

    response = namespace_controller.get_all_config(cluster, str(namespace), node)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace_name>/nodes/<node_list>/setconfig", methods=["POST"])
@session_manager.check_session_validity(session,['sys-admin'])
@cluster_manager.check_cluster_validity()
#@user_manager.allowed_for_roles("sys-admin")
def set_namespace_config(cluster, namespace_name, node_list, user, password):

    edit_parameters = request.form

    response = config_controller.set_namespace_config(cluster, namespace_name, node_list, edit_parameters, user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace>/sindexes/<sindex>/nodes/<node>/allstats",
    methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
def get_secondary_index_stats_from_node(cluster, namespace, sindex, node, user, password):

    stats_type = request.args.get("type", "all")

    response = namespace_controller.get_index_stats(cluster, str(namespace), str(sindex), node, stats_type, user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace>/sindexes",
    methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
def get_secondary_indexes_by_namespace(cluster, namespace,user, password):

    response = cluster_controller.get_indexes_by_namespace(cluster, str(namespace),
                                            user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace>/sets",
    methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
def get_sets_by_namespace(cluster, namespace, user, password):

    response = cluster_controller.get_sets_by_namespace(cluster, str(namespace),
                                        user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace>/storage",
    methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
def get_storage_info(cluster, namespace, user, password):

    response = namespace_controller.get_storage_info(cluster, str(namespace))
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/xdr/<xdr_port>/nodes/<node_list>", methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
def get_xdr_info(cluster, xdr_port, node_list, user, password):

    session_key = session['session']
    import sys
    #sys.stderr.write('\n\n\nIN FLASKAPP GET XDR INFO: ' + str(cluster))
    cluster_id = cluster.get_cluster_id()
    #sys.stderr.write('\n\nCLUSTER ID: ' + str(cluster_id))
    pl_password = session_manager.get_pl_password(session_key, cluster_id)

    response = xdr_controller.get_info(cluster, node_list, xdr_port,
                   user=user, password= password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/xdr/<xdr_port>/nodes/<node>/allstats",
    methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
def get_xdr_stats_from_node(cluster, xdr_port, node, user, password):

    session_key = session['session']
    cluster_id = cluster.get_cluster_id()
    pl_password = session_manager.get_pl_password(session_key, cluster_id)
    stats_type = request.args.get("type", "all")

    response = xdr_controller.get_stats(cluster, node, xdr_port, stats_type,
                   user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/xdr/<xdr_port>/nodes/<node>/allconfig",
    methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
def get_xdr_config(cluster, xdr_port, node, user, password):

    session_key = session['session']
    cluster_id = cluster.get_cluster_id()
    pl_password = session_manager.get_pl_password(session_key, cluster_id)
    response = xdr_controller.get_all_config(cluster, node, xdr_port,
                                user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/xdr/<xdr_port>/nodes/<node_list>/setconfig",
    methods=["POST"])
@session_manager.check_session_validity(session, set(["sys-admin"]))
@cluster_manager.check_cluster_validity()
def set_xdr_config(cluster, xdr_port, node_list, user, password):

    session_key = session['session']
    cluster_id = cluster.get_cluster_id()
    pl_password = session_manager.get_pl_password(session_key, cluster_id)
    edit_parameters = request.form
    response = config_controller.set_xdr_config(cluster, node_list, xdr_port, edit_parameters,
                                        user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/throughput", methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
def get_throughput(cluster, user, password):

    node_addresses = request.args.get('nodes',None)

    response = cluster_controller.get_throughput(cluster, node_addresses,
                                            user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/throughput_history", methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
def get_throughput_history(cluster, user, password):

    start_time = request.args.get('start_time', None)
    end_time = request.args.get('end_time', None)
    if start_time is not None:
        start_time = int(start_time)
    if end_time is not None:
        end_time = int(end_time)
    node_addresses = request.args.get('nodes', None)

    response = cluster_controller.get_throughput_history(cluster,
                                        node_addresses, start_time, end_time,
                                        user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/latency/<node_list>", methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
def get_latency_by_node(cluster, node_list, user, password):

    response = node_controller.get_latency(cluster, str(node_list),
                        user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/latency_history/<node>", methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
def get_latency_history_by_node(cluster, node, user, password):

    start_time = request.args.get('start_time', None)
    end_time = request.args.get('end_time', None)

    response = node_controller.get_latency_history(cluster, str(node), start_time, end_time,
                                            user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters", methods=["GET"])
def get_clusters_info():

    response = cluster_controller.get_clusters(cluster_manager)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/alerts", methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
def get_cluster_alerts(cluster, user, password):

    last_id = request.args.get('last_id', 0)

    response = cluster_controller.get_alerts(cluster, last_id, user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/jobs/nodes/<node>", methods=["GET"])
@session_manager.check_session_validity(session,['sys-admin'])
@cluster_manager.check_cluster_validity()
#@user_manager.allowed_for_roles("sys-admin")
def get_scan_jobs(cluster, node, user, password):

    response = udf_controller.get_jobs(cluster, node, user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/udfs", methods=["GET"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
def get_udfs(cluster, user, password):

    response = udf_controller.get_udfs(cluster, user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/nodes/<address>/switch_on", methods=["POST"])
@session_manager.check_session_validity(session,['sys-admin'])
@cluster_manager.check_cluster_validity()
#@user_manager.allowed_for_roles("sys-admin")
def switch_node_on(cluster, address, user, password):

    if ansible_present is True:

        if "username" in request.form and "password" in request.form:
            response = node_controller.change_node_status("on", cluster, address, request.form['username'], request.form['password'], usr=user, passwd=password or None)
        elif "username" in request.form:
            response = node_controller.change_node_status("on", cluster, address, request.form['username'], usr=user, passwd=password or None)
        else:
            response = node_controller.change_node_status("on", cluster, address, usr=user, passwd=password or None)
    else:
        response = dict(status="Failure", error="Ansible Not Found")
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/nodes/<address>/switch_off", methods=["POST"])
@session_manager.check_session_validity(session, ['sys-admin'])
@cluster_manager.check_cluster_validity()
#@user_manager.allowed_for_roles("sys-admin")
def switch_node_off(cluster, address, user, password):

    if ansible_present is True:

        if "username" in request.form and "password" in request.form:
            response = node_controller.change_node_status("off", cluster, address, request.form['username'], request.form['password'], usr=user, passwd=password or None)
        elif "username" in request.form:
            response = node_controller.change_node_status("off", cluster, address, request.form['username'], usr=user, passwd=password or None)
        else:
            response = node_controller.change_node_status("off", cluster, address, usr=user, passwd=password or None)
    else:
        response = dict(status="Failure", error="Ansible Not Found")
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/nodes/<address>/switch_xdr_on", methods=["POST"])
@session_manager.check_session_validity(session, ['sys-admin'])
@cluster_manager.check_cluster_validity()
#@user_manager.allowed_for_roles("sys-admin")
def switch_xdr_on(cluster, address, user, password):

    if ansible_present is True:

        if "username" in request.form and "password" in request.form:
            response = xdr_controller.change_xdr_status("on", cluster, address, request.form['username'], request.form['password'], usr=user, passwd=password or None)
        elif "username" in request.form:
            response = xdr_controller.change_xdr_status("on", cluster, address, request.form['username'], usr=user, passwd=password or None)
        else:
            response = xdr_controller.change_xdr_status("on", cluster, address, usr=user, passwd=password or None)
    else:
        response = dict(status="Failure", error="Ansible Not Found")
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/nodes/<address>/switch_xdr_off", methods=["POST"])
@session_manager.check_session_validity(session, ['sys-admin'])
@cluster_manager.check_cluster_validity()
#@user_manager.allowed_for_roles("sys-admin")
def switch_xdr_off(cluster, address, user, password):

    if ansible_present is True:

        if "username" in request.form and "password" in request.form:
            response = xdr_controller.change_xdr_status("off", cluster, address, request.form['username'], request.form['password'], usr=user, passwd=password or None)
        elif "username" in request.form:
            response = xdr_controller.change_xdr_status("off", cluster, address, request.form['username'], usr=user, passwd=password or None)
        else:
            response = xdr_controller.change_xdr_status("off", cluster, address, usr=user, passwd=password or None)
    else:
        response = dict(status="Failure", error="Ansible Not Found")
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/add_node", methods=["POST"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
def add_node(cluster, user, password):

    address = request.form['address']

    response = cluster_controller.add_node(cluster, address, user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/remove_node", methods=["POST"])
@session_manager.check_session_validity(session)
@cluster_manager.check_cluster_validity()
def remove_node(cluster, user, password):

    address = request.form['address']
    response = cluster_controller.remove_node(cluster, address)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/initiate_backup", methods=["POST"])
@session_manager.check_session_validity(session, ["sys-admin","read","read-write"])
@cluster_manager.check_cluster_validity()
#@user_manager.allowed_for_roles("read", "read-write")
def initiate_backup(cluster, user, password):

    if ansible_present is True:

        session_key = session['session']
        cluster_id = cluster.get_cluster_id()
        pl_password = session_manager.get_pl_password(session_key, cluster_id)

        import sys
        #sys.stderr.write('\nIn restore, GOT cluster id and pl password as: ' + str(cluster_id)+ '\t' + str(password))

        if "username" in request.form and "password" in request.form:
            response = cluster_controller.initiate_backup(cluster,
                                                   request.form['namespace'],
                                                   request.form['destination_node_address'],
                                                   request.form['destination_location'],
                                                   request.form['sets'],
                                                   request.form['only_metadata'],
                                                   request.form['scan_priority'],
                                                   request.form['terminate_on_change'],
                                                   request.form['username'],
                                                   request.form['password'],
                                                   usr=user,
                                                   passwd=password or None,
                                                   pl_password=pl_password
                                                   )
        elif "username" in request.form:
            response = cluster_controller.initiate_backup(cluster,
                                                   request.form['namespace'],
                                                   request.form['destination_node_address'],
                                                   request.form['destination_location'],
                                                   request.form['sets'],
                                                   request.form['only_metadata'],
                                                   request.form['scan_priority'],
                                                   request.form['terminate_on_change'],
                                                   request.form['username'],
                                                   usr=user,
                                                   passwd=password or None,
                                                   pl_password=pl_password                                                )
        else:
            response = cluster_controller.initiate_backup(cluster,
                                                   request.form['namespace'],
                                                   request.form['destination_node_address'],
                                                   request.form['destination_location'],
                                                   request.form['sets'],
                                                   request.form['only_metadata'],
                                                   request.form['scan_priority'],
                                                   request.form['terminate_on_change'],
                                                   usr=user,
                                                   passwd=password or None,
                                                   pl_password=pl_password
                                                   )
    else:
        response = dict(status="Failure", error="Ansible Not Found")
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/get_backup_progress", methods=["GET"])
@session_manager.check_session_validity(session,["read", "read-write"])
@cluster_manager.check_cluster_validity()
#@user_manager.allowed_for_roles("read", "read-write")
def get_backup_progress(cluster, user, password):

    if ansible_present is True:
        response = cluster_controller.get_backup_progress(cluster)
    else:
        response = dict(status="Failure", error="Ansible Not Found")
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/get_successful_backups", methods=["GET"])
@session_manager.check_session_validity(session,["read", "read-write"])
@cluster_manager.check_cluster_validity()
#@user_manager.allowed_for_roles("read", "read-write")
def get_successful_backups(cluster, user, password):

    response = cluster_controller.get_successful_backups(cluster)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/get_available_backups", methods=["POST"])
@session_manager.check_session_validity(session,["read","read-write"])
@cluster_manager.check_cluster_validity()
#@user_manager.allowed_for_roles("read", "read-write")
def get_available_backups(cluster, user, password):

    if ansible_present is True:
        response = cluster_controller.get_available_backups(cluster,
        request.form['source_node_address'], request.form['source_location'],
        request.form['username'], request.form['password']
        )
    else:
        response = dict(status="Failure", error="Ansible Not Found")
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/initiate_restore", methods=["POST"])
@session_manager.check_session_validity(session,["sys-admin","read-write"])
@cluster_manager.check_cluster_validity()
#@user_manager.allowed_for_roles("read-write")
def initiate_restore(cluster, user, password):

    if ansible_present is True:

        session_key = session['session']
        cluster_id = cluster.get_cluster_id()
        pl_password = session_manager.get_pl_password(session_key, cluster_id)

        import sys
        #sys.stderr.write('\nIn restore, GOT cluster id and pl password as: ' + str(cluster_id)+ '\t' + str(password))
        if "username" in request.form and "password" in request.form:
            response = cluster_controller.initiate_restore(cluster,
                                                   request.form['source_node_address'],
                                                   request.form['source_location'],
                                                   request.form['threads'],
                                                   request.form['namespace'],
                                                   request.form['missing_records_only'],
                                                   request.form['ignore_generation_num'],
                                                   request.form['username'],
                                                   request.form['password'],
                                                   usr=user,
                                                   passwd=password or None,
                                                   pl_password=pl_password
                                                   )
        elif "username" in request.form:
            response = cluster_controller.initiate_restore(cluster,
                                                   request.form['source_node_address'],
                                                   request.form['source_location'],
                                                   request.form['threads'],
                                                   request.form['namespace'],
                                                   request.form['missing_records_only'],
                                                   request.form['ignore_generation_num'],
                                                   request.form['username'],
                                                   usr=user,
                                                   passwd=password or None,
                                                   pl_password=pl_password
                                                   )
        else:
            response = cluster_controller.initiate_restore(cluster,
                                                   request.form['source_node_address'],
                                                   request.form['source_location'],
                                                   request.form['threads'],
                                                   request.form['namespace'],
                                                   request.form['missing_records_only'],
                                                   request.form['ignore_generation_num'],
                                                   usr=user,
                                                   passwd=password or None,
                                                   pl_password=pl_password
                                                   )
    else:
        response = dict(status="Failure", error="Ansible Not Found")
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/get_restore_progress", methods=["GET"])
@session_manager.check_session_validity(session, ["read-write"])
@cluster_manager.check_cluster_validity()
#@user_manager.allowed_for_roles("read-write")
def get_restore_progress(cluster, user, password):

    if ansible_present is True:

        response = cluster_controller.get_restore_progress(cluster, user=user, password=password or None)
    else:
        response = dict(status="Failure", error="Ansible Not Found")
    return json.dumps(response)


@app.after_request
def after_request(response):

    response.headers.add('Access-Control-Allow-Origin', '*')
    response.headers.add('Access-Control-Allow-Credentials', 'true')
    response.headers.add('Access-Control-Max-Age', 10)
    response.headers.add('Access-Control-Allow-Methods', 'GET')
    response.headers.add('Cache-Control',
        'must-revalidate, post-check=0, pre-check=0')
    content_type = response.headers.get('Content-Type')
    if content_type.find("text/html") != -1:
        try:
            homepage = response.headers['amc-page']
        except:
            response.headers['Content-Type'] = "application/json"
            response.headers['Cache-Control'] += ', no-cache'
            return response
    expiry_time = datetime.datetime.utcnow() + datetime.timedelta(30)
    response.headers["Expires"] = expiry_time.strftime("%a, %d %b %Y %H:%M:%S GMT")
    return response

@app.route("/aerospike/service/clusters/<cluster>/get_user_roles", methods=["GET"])
#@login_required
@session_manager.check_session_validity(session)
def get_user_roles(cluster, user, password):

    session_key = session['session']
    roles = session_manager.get_user_roles_by_cluster_id(session_key, cluster)
    if roles is not None:
        roles = list(roles)
    return json.dumps(dict(rolelist=roles))


app.debug = True
app.config["REMEMBER_COOKIE_DURATION"] = datetime.timedelta(days=14)

# Configuring an application with the flask-login
# refer https://flask-login.readthedocs.org/en/latest/
#login_manager.init_app(app)

e = eventlet.spawn(background_tasks.perform_scheduled_tasks, cluster_manager, app)
try:
    f = open('/opt/amc/amc_version', 'r')
    amc_version = f.read()
    f.close()
except:
    amc_version = "Error reading AMC version"
sys.stderr.write("AMC Version: %s\n" % amc_version)

if __name__ == "__main__":

    #Change the duration of how long the Remember Cookie is valid on the users
    #computer.  This can not really be trusted as a user can edit it.
    app.config["REMEMBER_COOKIE_DURATION"] = datetime.timedelta(days=14)

    #Tell the login manager where to redirect users to display the login page
    #login_manager.login_view = "/login/"

    #Setup the login manager.
    login_manager.init_app(app)

    app.run(host='0.0.0.0', port=5001)

