"""
Copyright 2008-2014 by Aerospike, Inc. All rights reserved.
THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE. THE COPYRIGHT NOTICE
ABOVE DOES NOT EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION.
"""


# -*- coding: utf-8 *-*


import functools
import json
from libs import base
from flask import current_app
from flask_kvsession import KVSessionExtension
from simplekv.memory import DictStore


store = DictStore()
KVSessionExtension(store, current_app)

class SessionManager(object):

    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(SessionManager, cls).__new__(
                                cls, *args, **kwargs)
            cls._instance.__init__()
        return cls._instance

    def __init__(self):
        self._sessions = dict()

    def create_session(self, session_key):
        self._sessions[session_key] = dict()

    def delete_session(self, session_key):
        #session['session'] = ''
        import sys
        import time
        import datetime
        ts = time.time()
        st = datetime.datetime.fromtimestamp(ts).strftime('%Y-%m-%d %H:%M:%S')
        sys.stderr.write('\n'+ str(st) + ' : Logging Out.....')
        del(self._sessions[session_key])

    def get_clusters_from_session(self, session_key):

        list_dict = []
        for cluster_id in self._sessions[session_key]:
            temp_dict = dict()
            temp_dict['username'] = self._sessions[session_key][cluster_id]['user']
            temp_dict['cluster_name'] = self._sessions[session_key][cluster_id]['cluster_name']
            roles = self._sessions[session_key][cluster_id]['roles']
            temp_dict['roles'] = roles if roles is None else list(roles)
            temp_dict['seed_node'] = self._sessions[session_key][cluster_id]['seed_node']
            temp_dict['cluster_id'] = cluster_id
            list_dict.append(temp_dict)
        return list_dict

        """temp_dict = self._sessions[session_key]
        for key in self._sessions[session_key]:
            for cluster_key in self._sessions[session_key][key]:
                del temp_dict[session_key][key][cluster_key]['password']
                del temp_dict[session_key][key][cluster_key]['roles']
        return temp_dict"""

    def add_cluster(self, session_key, node_address, cluster_id, cluster_name, user=None, password=None, pl_password = None):

        import sys
        ip, port = node_address.split(":")
        #sys.stderr.write('\n clusters in session:' + str(self._sessions[session_key]))
        user_roles = base.get_cluster_userroles(ip, port,user=user, password=password)
        if user_roles is not None:
            user_roles = set(user_roles)
        self._sessions[session_key][cluster_id] = dict(user= user, password= password,
             roles = user_roles, cluster_name = cluster_name,
             seed_node = node_address, pl_password = pl_password)
        #sys.stderr.write('\n clusters in session:' + str(self._sessions[session_key]))
        #temp_dict = self._sessions[session_key][cluster_id]
        #sys.stderr.write('\n user:' + str(temp_dict['user']) + '\t password:' + str(temp_dict['password']) + '\t cluster name: ' + str(temp_dict['cluster_name']) + '\t seed node: ' + str(temp_dict['seed_node']) +  '\troles: ' + str(temp_dict['roles'])+ '\t plain password: ' + str(temp_dict['pl_password']))

    def remove_cluster(self, session_key, cluster_id):

        #import sys
        #sys.stderr.write('\nSession before removing cluster:  ' + str(self._sessions[session_key]))
        del(self._sessions[session_key][cluster_id])
        if cluster_id not in self._sessions[session_key]:
            #sys.stderr.write('\nREMOVED CLUSTER: ' + str(cluster_id))
            #sys.stderr.write('\nSession after removing cluster:  ' + str(self._sessions[session_key]))
            return True
        return False

    def get_user_roles_by_cluster_id(self, session_key, cluster_id):
        temp_dict = self._sessions[session_key][cluster_id]
        return temp_dict['roles']

    def are_roles_matching(self, session_key, cluster_id, roles):

        user_roles = self.get_user_roles_by_cluster_id(session_key, cluster_id)
        if user_roles is None:
            return True
        common_roles = user_roles.intersection(roles)
        return True if len(common_roles) != 0 else False

    def get_pl_password(self, session_key, cluster_id):
        temp_dict = self._sessions[session_key][cluster_id]
        return temp_dict['pl_password']

    def is_valid_session(self, session_key):
        return session_key in self._sessions

    def is_cluster_present(self, session_key, node_address=None, cluster_name=None):
        if node_address is not None:
            for cluster_id, data in self._sessions[session_key].items():
                if data['seed_node'] == node_address:
                    if cluster_name is not None:
                        self._sessions[session_key][cluster_id]['cluster_name'] = cluster_name
                    return cluster_id
        elif cluster_name is not None:
            for cluster_id, data in self._sessions[session_key].items():
                if data['cluster_name'] == cluster_name:
                    return cluster_id
        return False

    def is_cluster_id_present(self, session_key, cluster_id):

        if cluster_id in self._sessions[session_key]:
            return True
        return False

    def get_user(self, session_id, cluster_key):

        #if cluster_key in self._sessions[session_id]:
            #sys.stderr.write('\ncluster key present!! ' + str(cluster_key) + '\n' + str(self._sessions[session_id][cluster_key]))
        #else:
            #sys.stderr.write('\ncluster key ABSENT!!')
        #sys.stderr.write('\n In get user session manager, user: '
           # + str(self._sessions[session_id][cluster_key]['user']))
        return self._sessions[session_id][cluster_key]['user']

    def get_cluster_name(self, session_id, cluster_key):
        return self._sessions[session_id][cluster_key]['cluster_name']

    def update_cluster_name(self, session_key, cluster_id, cluster_name):
        #import sys
        #sys.stderr.write('\n\n\nin update_cluster NAME')
        self._sessions[session_key][cluster_id]['cluster_name'] = cluster_name


    def check_session_validity(self, session_obj, roles=None):

        def decorator(fn):

            @functools.wraps(fn)
            def f(*args, **kwargs):

                """if roles is not None:
                        import sys
                        sys.stderr.write('\nRoles are: ' + str(roles))"""
                session_key = session_obj.get('session', None)
                cluster_id = kwargs.get('cluster', None)
                if session_key is not None and cluster_id is None:
                    return fn(*args, **kwargs)
                elif session_key is not None and cluster_id is not None:
                    if cluster_id not in self._sessions[session_key]:
                        return json.dumps(dict(error="Invalid cluster id : %s" % kwargs["cluster"]))
                    temp_dict = self._sessions[session_key][cluster_id]
                    kwargs['user'] = temp_dict['user']
                    kwargs['password'] = temp_dict['password']

                    if roles is None:
                        return fn(*args, **kwargs)
                    elif self.are_roles_matching(session_key, cluster_id, roles):
                        return fn(*args, **kwargs)
                    else:
                        return json.dumps(dict(
                            error="You do not have enough permissions to access it."))
                elif session_key is None:
                    return json.dumps(dict(error="Invalid session : %s" % session_key))
                elif cluster_id is None:
                    return json.dumps(dict(error="Invalid cluster id : %s" % kwargs["cluster"]))

            return f
        return decorator



